@@ -1,10 +1,9 @@
 import datetime
+import numpy as np
+
 from typing import Callable, Union
 
-import numpy as np
 from ..types import Interval
-from scipy.interpolate import interp1d
-
 from ..utils import singledispatch
 from .base import BaseIndex
 
@@ -23,10 +22,7 @@ def interpolater(x, xs, ys, kind="linear"):
 @interpolater.register(int)
 def interpolate_number(x, xs, ys, kind="linear"):
     if isinstance(ys[0], (float, int)):
-        func = interp1d(
-            xs, ys, fill_value=(ys[0], ys[-1]), bounds_error=False, kind=kind
-        )
-        return func(x).item()
+        return np.interp(x, xs, ys)
     return nn_interpolate(x, xs, ys)
 
 
@@ -62,12 +58,12 @@ class InterpolatingIndex(BaseIndex):
         return self.extrapolate
 
     def reduce(self, docs, label):
-     
+
         if isinstance(label, list):
             return [d for val in label for d in self.reduce(docs, val)]
 
         label = self.validate_label(label)
-        
+
         if not docs or label is None:
             return docs
 
@@ -78,7 +74,7 @@ class InterpolatingIndex(BaseIndex):
         # just convert all datetimes to timestamps to avoid complexity
         # FIXME: maybe properly handle timezones instead
         xs = [xi.timestamp() if isinstance(xi, datetime.datetime) else xi for xi in xs]
-        
+
         if len(docs) == 1:
             new_document = docs[0]
         else:
@@ -104,4 +100,4 @@ class InterpolatingIndex(BaseIndex):
         if left is None or right is None:
             return []
         iv_class = Interval[type(left)]
-        return [iv_class(left=left, right=right)]
\ No newline at end of file
+        return [iv_class(left=left, right=right)]
