@@ -8,9 +8,9 @@ from ..types import Interval, TimeInterval
 
 
 class IntervalIndex(BaseIndex):
-    
+
     __slots__ = BaseIndex.__slots__ + ("closed",)
-    
+
     def __set_name__(self, owner, name):
         super().__set_name__(owner, name)
 
@@ -24,21 +24,19 @@ class IntervalIndex(BaseIndex):
         self.closed = closed
 
     def label_options(self, query):
-        '''Collect all valid intervals
-        '''
+        """Collect all valid intervals"""
         labels = query.unique(self.name)
         if not labels:
             return []
         if isinstance(labels[0], dict):
             labels = [self.field.type_(**label) for label in labels]
         labels = sorted(labels, key=lambda x: x.left)
-        
+
         ivs = labels[:1]
         for iv in labels[1:]:
             # touching intervals can be combined
             if iv.left == ivs[-1].right:
-                ivs[-1] = iv.__class__(left=ivs[-1].left,
-                                          right=iv.right)
+                ivs[-1] = Interval[ivs[-1].left,iv.right]
             else:
                 ivs.append(iv)
-        return ivs
\ No newline at end of file
+        return ivs
